"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tsRouteParser_1 = require("./tsRouteParser");
var fs = require("fs-extra");
var pathModule = require("path");
var commander = require("commander");
var program = commander
    .description("Core util to generate leaf-ts required resources")
    .option("--create-route-target <path>", "create generated root ts api by search for root-ts project route ")
    .option("-o, --output <output>", "out to file to target file")
    .option("--generate-route <path/to/route.ts>", "Generate route file by routes")
    .option("--sync-route <path/to/route/folders>", "Sync route spec to route files")
    .option("--route-folder <routeFolder>", "Target folder to sync route")
    .option("--guess", "Smart guess sync, enable with risk in mind")
    .parse(process.argv);
if (program["createRouteTarget"]) {
    var path = program["createRouteTarget"];
    var parser = new tsRouteParser_1.TSRouteParser(path.split(","));
    var infos = parser.parse();
    var generator = new tsRouteParser_1.RouteAPIGenerator(infos);
    var content = generator.build();
    if (program.output) {
        var orig = fs.readFileSync(program.output, "utf8");
        if (orig !== content) {
            fs.writeFileSync(program.output, content);
        }
    }
    else {
        console.log(content);
    }
}
if (program["generateRoute"]) {
    var routeSpec_1 = program["generateRoute"];
    var parser_1 = new tsRouteParser_1.TSRouteParser(routeSpec_1.split(","));
    var infos = parser_1.parse();
    if (program["routeFolder"] || !program["syncRoute"]) {
        var routeFolder_1 = program["routeFolder"] || program["syncRoute"];
        infos.forEach(function (info) {
            var syncer = new tsRouteParser_1.RouteAPISyncer(parser_1, info);
            var filename = info.name + ".ts";
            var path = pathModule.join(routeFolder_1, filename);
            if (fs.existsSync(path)) {
                console.log(path, "exists, skip generating...");
                return;
            }
            //console.log(path)
            //console.log(syncer.buildFile(routeSpec, path))
            fs.writeFileSync(path, syncer.buildFile(routeSpec_1, path));
        });
    }
    else {
        console.error("Missing --route-folder");
    }
}
if (program["syncRoute"]) {
    var path = program["syncRoute"];
    var parser_2 = new tsRouteParser_1.TSRouteParser(path.split(","));
    var infos = parser_2.parse();
    infos.map(function (info) {
        var syncer = new tsRouteParser_1.RouteAPISyncer(parser_2, info);
        // Don't async abstract we only do the sync
        if (info.class.abstract)
            return;
        var content = syncer.merge({
            path: info.seed,
            guess: true
        });
        console.log(info.seed, "synced");
        var orig = fs.readFileSync(info.seed, "utf8");
        if (orig != content) {
            fs.writeFileSync(info.seed, content);
        }
    });
}
//# sourceMappingURL=main.js.map