import { TSRouteParser, RouteAPIGenerator, RouteAPISyncer } from "./tsRouteParser"
import * as fs from "fs-extra"
import * as pathModule from "path"
import * as ts from "typescript"
import * as wrench from "wrench"
import * as commander from "commander"
import * as colors from "colors"
let program = commander
    .description("Core util to generate leaf-ts required resources")
    .option("--create-route-target <path>", "create generated root ts api by search for root-ts project route ")
    .option("-o, --output <output>", "out to file to target file")
    .option("--generate-route <path/to/route.ts>", "Generate route file by routes")
    .option("--sync-route <path/to/route/folders>", "Sync route spec to route files")
    .option("--route-folder <routeFolder>", "Target folder to sync route")
    .option("--guess", "Smart guess sync, enable with risk in mind")
    .parse(process.argv)
if (program["createRouteTarget"]) {
    let path = program["createRouteTarget"]
    let parser = new TSRouteParser(path.split(","))
    let infos = parser.parse()
    let generator = new RouteAPIGenerator(infos)
    let content = generator.build()
    if (program.output) {
        let orig = fs.readFileSync(program.output, "utf8")
        if (orig !== content) {
            fs.writeFileSync(program.output, content)
        }
    } else {
        console.log(content)
    }
}

if (program["generateRoute"]) {
    let routeSpec = program["generateRoute"]
    let parser = new TSRouteParser(routeSpec.split(","))
    let infos = parser.parse()
    if (program["routeFolder"] || !program["syncRoute"]) {
        let routeFolder = program["routeFolder"] || program["syncRoute"]
        infos.forEach(info => {
            let syncer = new RouteAPISyncer(parser, info)
            let filename = info.name + ".ts"
            let path = pathModule.join(routeFolder, filename)
            if (fs.existsSync(path)) {
                console.log(path, "exists, skip generating...")
                return
            }
            //console.log(path)
            //console.log(syncer.buildFile(routeSpec, path))
            fs.writeFileSync(path, syncer.buildFile(routeSpec, path))
        })
    } else {
        console.error("Missing --route-folder")
    }
}

if (program["syncRoute"]) {
    let path = program["syncRoute"]
    let parser = new TSRouteParser(path.split(","))
    let infos = parser.parse()
    infos.map(info => {
        let syncer = new RouteAPISyncer(parser, info)
        // Don't async abstract we only do the sync
        if (info.class.abstract) return
        let content = syncer.merge({
            path: info.seed,
            guess: true
        })
        console.log(info.seed, "synced")
        let orig = fs.readFileSync(info.seed, "utf8")
        if (orig != content) {
            fs.writeFileSync(info.seed, content)
        }
    })
}
