import * as ts from "typescript"
export class TempFileHost implements ts.LanguageServiceHost {
    constructor(public readonly file: string) {
    }
    getScriptFileNames() {
        return ["main.ts"]
    }
    getScriptVersion(fileName) {
        return "latest"
    }
    getScriptSnapshot(fileName) {
        if (fileName == "main.ts") return ts.ScriptSnapshot.fromString(this.file)
        return undefined
    }
    getCurrentDirectory() {
        return process.cwd()
    }
    getCompilationSettings() {
        return {}
    }
    getDefaultLibFileName(options) {
        return ts.getDefaultLibFilePath(options)
    }
    fileExists = (name) => name == "main.ts"
    readFile = (name) => name == "main.ts" && this.file || undefined
    readDirectory = () => []
};

export function format(content: string) {
    let host = new TempFileHost(content)
    let language = ts.createLanguageService(host, ts.createDocumentRegistry())
    let option = {
        convertTabsToSpaces: true,
        insertSpaceAfterCommaDelimiter: true,
        insertSpaceAfterKeywordsInControlFlowStatements: true,
        insertSpaceBeforeAndAfterBinaryOperators: true,
        newLineCharacter: "\n",
        indentStyle: ts.IndentStyle.Smart,
        indentSize: 4,
        tabSize: 4
    }
    let finalText = content
    const textChanges = language.getFormattingEditsForDocument("main.ts", option)
    for (const textChange of textChanges.sort((a, b) => b.span.start - a.span.start)) {
        textChange
        const { span } = textChange;
        finalText = finalText.slice(0, span.start) + textChange.newText
            + finalText.slice(span.start + span.length);
    }
    return finalText

}
