import * as mongodb from "mongodb"
import { Service } from "../lib/service"

import { MemoryLockService } from "../service/memoryLockService"
import { MongodbService } from "../service/mongodbService"
import { ExpressService } from "../service/expressService"
import { RouteSessionService } from "../service/routeSessionService"
import { TTLSessionStorage } from "../service/ttlSessionStorage"
import { IncrementalIdService } from "../service/incrementalIdService"

import { UploadService } from "../service/uploadService"
import { BlobService } from "../service/blobService"

export type AllServices = {
    MemoryLockService: MemoryLockService
    MongodbService: MongodbService
    ExpressService: ExpressService
    RouteSessionService: RouteSessionService
    TTLSessionStorage: TTLSessionStorage
    IncrementalIdService: IncrementalIdService
    UploadService: UploadService
    BlobService: BlobService
}

export abstract class KeyValueStorage extends Service<Type.Root.KeyValueStorage.Settings> {
    constructor(settings: Type.Root.KeyValueStorage.Settings) {
        super({
            kvs: {
                collection: "kvs"
            }
        }, settings)
    }
    services: {
        MongodbService: MongodbService
    }
    readonly name = "KeyValueStorage"
    readonly dependencies = ["MongodbService"]
    abstract set<T>(k: string, v: T, callback?: Callback<null>)
    abstract get<T>(k: string, callback: Callback<T>)
}

export abstract class EndlessService extends Service<Type.Root.EndlessService.Settings> {
    readonly name = "EndlessService"
    dependencies = []
    services: {
    }
    constructor(settings: Type.Root.EndlessService.Settings) {
        super({
            endless: {
                count: 10
            },
        }, settings)
    }
    abstract query<K, T = any>(option: Type.Root.EndlessService.EndlessQuery<T> & {
        collection: mongodb.Collection
    }, callback: Callback<K[]>)
}

export abstract class MailService extends Service<Type.Root.MailService.Settings>{
    readonly name = "MailService"
    dependencies = []
    services: {}
    constructor(settings: Type.Root.MailService.Settings) {
        super({
            mail: {
                provider: null,
                sendgrid: {
                    apiKey: null
                }
            },
        }, settings)
    }
    abstract send(option: {
        type?: "sendgrid"
        from: Type.Root.MailService.EmailAddress
        to: Type.Root.MailService.EmailAddress
        mail: Type.Root.MailService.Mail
    }, callback: Callback<null>)
}

export abstract class GracefulShutdownService extends Service<Type.Root.GracefulShutdownService.Settings> {
    readonly name = "GracefulShutdownService"
    readonly dependencies = []
    services: {}
    constructor(settings: Type.Root.GracefulShutdownService.Settings) {
        super({
            gracefulShutdown: {
                timeout: 1000
            }
        }, settings)
    }
    abstract graceful(option: {
        name: string
        timeout?: number
    }, callback: (done: Function) => void)

}

export abstract class PersistentService extends Service<Type.Root.PersistentService.Settings> {
    readonly name = "PersistentService"
    readonly dependencies = ["GracefulShutdownService"]
    services: {
        GracefulShutdownService: GracefulShutdownService
    }
    constructor(settings: Type.Root.PersistentService.Settings) {
        super({
            persistent: {
                storage: "/tmp",
                storageName: "persistent.json"
            }
        }, settings)
    }
    abstract track(obj: Type.Root.PersistentService.Persistentable)
    abstract restore(obj: Type.Root.PersistentService.Persistentable)
}

