type Callback<T, E = Error> = (err?: E, data?: T) => void
type GUID = string
declare namespace Type {
    namespace Root {
        namespace GracefulShutdownService {
            type Settings = {
                gracefulShutdown: {
                    // default timeout for a single job
                    timeout: number
                }
            }
        }
        namespace KeyValueStorage {
            export type Settings = {
                kvs: {
                    collection: string
                }
            }
        }

        namespace BlobService {
            export type Settings = {
                blob: {
                    route?: string
                    storage: string
                }
            }
        }

        namespace ExpressService {
            export type Settings = {
                http?: {
                    port?: number
                    host?: string
                    offline?: boolean
                    limit: {
                        jsonInKb?: number
                        bodyInKb?: number
                    }
                    staticResources?: {
                        path: string
                        route: string
                    }[]
                }
            }
        }

        namespace RouteSessionService {
            export type Settings = {
                http?: {
                    session?: {
                        secret?: string
                        maxAge?: number
                        resave?: boolean
                        saveUninitialized?: boolean
                    }
                }
            }
        }

        namespace EndlessService {
            export type Settings = {
                endless?: {
                    count?: number
                }
            }
            export type EndlessQueryMeta = {
                offset?: number,
                count?: number,
            }
            export type EndlessQuery<T = any> = {
                offset: number,
                count: number,
                criteria?: {
                    [K in keyof T]?: any
                }
                sortField?: string,
                sortOrder?: number
            }
        }
        namespace MailService {
            type Settings = {
                mail: {
                    provider: "sendgrid"
                    sendgrid?: {
                        apiKey: string
                    }
                }
            }
            type EmailAddress = string
            type HTML = string
            type Mail = {
                title: string
                name: string
                content: HTML
                fallbackText?: string
                fromName?: string
                toName?: string
            }
        }

        namespace PersistentService {
            type Settings = {
                persistent: {
                    storage: string
                    storageName: string
                }
            }
            interface Persistentable {
                name: string
                toJSON(): any
                fromJSON(data: any)
            }
        }
    }
}
